import ptah

View = ptah.Permission('ptah:View', 'View')
AddContent = ptah.Permission('ptah-cms:AddContent', 'Add content')
ModifyContent = ptah.Permission('ptah-cms:EditContent', 'Modify content')
DeleteContent = ptah.Permission('ptah-cms:DeleteContent', 'Delete content')
ShareContent = ptah.Permission('ptah-cms:ShareContent', 'Share content')
